/*
This is a module for Litestep which let you define hotspots for
executing bang commands or launching programs

Copyright (C) 2003 Eric Moore
Copyright (C) 2000 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#pragma comment( lib, "lsapi.lib" )

#define STRICT
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <stdlib.h>
#include <strsafe.h>
#include <litestep/lsapi/lsapi.h>
#include <litestep/utility/macros.h>
#include <litestep/utility/wharfdata.h>
#include "ckHotspots.h"

HWND g_HotspotHWND = NULL;


////////////////////////////////
//helper functions
//change so that coord will not be < 0
//void AdjustCoordinate(int *coord, int inc);

// turn on and off the hotspot system, when on, the system
// will check for mouse movement and stuff
void SystemOn(void);
void SystemOff(void);

// this function will adjust to turn system on or off
// depend on whether all hotspots or activated or deactivated
void SystemAdjust(void);


void SetMouseHook(void);
void RemoveMouseHook(void);
BOOL LoadHookSettings();
void UnloadHookSettings();


///////////////////////////////////
//hook functions
//our mouse call back function
//LRESULT CALLBACK MouseProc(int nCode, WPARAM wParam, LPARAM lParam);
//setup the mouse hook	
//void SetMouseHook(void);
//void RemoveMouseHook(void);
HINSTANCE g_HookInstance = NULL;
SETMOUSEHOOK SetMouseHookFunc = NULL;


//////////////////////////////////
//bang functions
//usual toggle functions
void BangOn(HWND caller, LPCSTR args);
void BangOff(HWND caller, LPCSTR args);
void BangToggle(HWND caller, LPCSTR args);

//dynamic add 
void BangAdd(HWND caller, LPCSTR args);
void BangRemove(HWND caller, LPCSTR args);

//Added by Eric Moore 2003-04-27
void BangAdjust(HWND caller, LPCSTR args);


// global variables
HINSTANCE g_hInstance = NULL;
UINT g_Timer = 0;
HotspotT g_Hotspots[MAX_HOTSPOTS];
int g_Count = 0;
int g_LastSpot = -1; /* hotspot the mouse was last in or -1 */ 
//HWND g_HotspotHWND = NULL;
POINT g_LastPt= {0, 0};
UINT Msgs[] = { LM_GETREVID, 0};
//true means all spots checking on,  false means all spot checking off
//-1 means that some are on some are off
BOOL g_Status = -1; 
BOOL g_Version1Style = FALSE;
BOOL g_MouseHookStyle = FALSE;

//get screen dimention
RECT g_rScreen;

static const char rcsRevision[] = "$Revision: 2.3.2 (Chaku, Hollow, RabidCow)$";
static const char rcsId[] = "$Id: ckHotspots v2.3.2 2003-11-21 Chao-Kuo Lin, Hollow, RabidCow $";

//
void DoPoint(HWND hwnd, POINT pt)
{
  int i;

  if (!g_Status) /* all checking is off */ 
    return;

	/* if anything changed    */ 
	if(g_LastPt.x != pt.x || g_LastPt.y != pt.y)
	{
    /* set first to prevent looping */ 
		g_LastPt = pt;

		/* we entered already, if mouse out, then we call the out action */ 
		if(g_LastSpot >= 0 && !PtInRect(&(g_Hotspots[g_LastSpot].spot), pt) )
		{
      /* reset first to prevent looping */ 
      i = g_LastSpot;
      g_LastSpot = -1;

			if(g_Hotspots[i].stat == TRUE)
				LSExecute(hwnd, g_Hotspots[i].out_action, SW_SHOWDEFAULT);
		}

		if(g_LastSpot < 0)
		{
			for(i=0; i<g_Count; ++i)
			{
				if((g_Hotspots[i].stat == TRUE) && PtInRect(&(g_Hotspots[i].spot), pt))
				{
					g_LastSpot = i;
					LSExecute(hwnd, g_Hotspots[i].in_action, SW_SHOWDEFAULT);
					break; /* we don't do overlap ones */ 
				}
			}
		}
	}
}

/////////////////////////////
// windows callback function
LRESULT CALLBACK WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
switch (uMsg) 
{
  POINT pt;
  volatile static unsigned nNested = 0;
	
	// user defined window message which is sent from our mouse hook
	// wParam contains the y coordinate while lParam contains the x coordinate
	case WM_CKMOUSEHOOKMSG:
    pt.x = lParam;
    pt.y = wParam;
    DoPoint(hwnd, pt);
    break;

	case WM_TIMER:
    if (GetCursorPos(&pt))
      DoPoint(hwnd, pt);
    break;

	case LM_GETREVID: 
		{
			PTSTR buffer = (PTSTR)lParam;
			
			if (wParam == 0)
			{
				StringCchCopy(buffer, MAX_LINE_LENGTH, "ckHotpots: ");
				StringCchCat(buffer, MAX_LINE_LENGTH, &rcsRevision[11]);
				buffer[strlen(buffer)-1] = 0;
			}
			else if (wParam == 1)
			{
				StringCchCopy(buffer, MAX_LINE_LENGTH, &rcsId[5]);
				buffer[strlen(buffer)-1] = 0;
			} else
			{
				StringCchCopy(buffer, MAX_LINE_LENGTH, "");
			}
			return strlen(buffer);
		}
		break;
  

	default:
		return DefWindowProc (hwnd, uMsg, wParam, lParam);
	}

	return 0;
}


////////////////////////////////
// Bang callback functions

typedef void (*HotspotModifier)(HotspotT *, void *);

/* perform some action on all matching hotspots (split 2003-11-21, blame RabidCow) */ 
int ForEachHotspot(LPCSTR sName, HotspotModifier pFunc, void *pData)
{
  int i;
  int nProcessed = 0;

  if (sName && *sName) /* non-null name */ 
  {
    for (i=0; i<g_Count; ++i)
    {
      if (g_Hotspots[i].stat == SLOT_EMPTY) continue;

      if (match(sName, g_Hotspots[i].name))
      {
        (*pFunc)(&g_Hotspots[i], pData);
        ++nProcessed;
      }
    }
  }
  else
  {
    for (i=0; i<g_Count; ++i)
    {
      if (g_Hotspots[i].stat == SLOT_EMPTY) continue;

      (*pFunc)(&g_Hotspots[i], pData);
      ++nProcessed;
    }
  }

  return nProcessed;
}


//BangOn
//	args will be the name of the spot to turn on,
//	if no args or arges len == 0, then it's all the area we are toggling
void HotspotOn(HotspotT *pSpot, void *NotUsed) { pSpot->stat = TRUE; }
void BangOn(HWND caller, LPCSTR args)
{	
  ForEachHotspot(args, &HotspotOn, NULL);
	SystemAdjust();
}

void HotspotOff(HotspotT *pSpot, void *NotUsed) { pSpot->stat = FALSE; }
void BangOff(HWND caller, LPCSTR args)
{
  ForEachHotspot(args, &HotspotOff, NULL);
	SystemAdjust();
}

void HotspotToggle(HotspotT *pSpot, void *NotUsed) { pSpot->stat = pSpot->stat ? FALSE : TRUE; }
void BangToggle(HWND caller, LPCSTR args)
{
  ForEachHotspot(args, &HotspotToggle, NULL);
	SystemAdjust();
}

void BangAdd(HWND caller, LPCSTR args)
{
	int i;

	for(i=0; i<g_Count; ++i)
	{
		if(g_Hotspots[i].stat == SLOT_EMPTY) break;
	}

	if(i >= MAX_HOTSPOTS) 
	//change to a warning box
		MessageBox(NULL, "Cannot add anymore spots!", "ckHotspots", MB_OK);
	else
	{
		int count = 0;
		char *lpszBuffers[10];
		char token1[MAX_LINE_LENGTH], token2[MAX_LINE_LENGTH], token3[MAX_LINE_LENGTH], token4[MAX_LINE_LENGTH], 
			token5[MAX_LINE_LENGTH], token6[MAX_LINE_LENGTH], token7[MAX_LINE_LENGTH], token8[MAX_LINE_LENGTH];
		token1[0] = token2[0] = token3[0] = token4[0] = 
		token5[0] = token6[0] = token7[0] = token8[0] = '\0';

		lpszBuffers[0] = token1;	// x1
		lpszBuffers[1] = token2;	// y1
		lpszBuffers[2] = token3;	// width
		lpszBuffers[3] = token4;	// height
		lpszBuffers[4] = token5;	// mouse in action
		lpszBuffers[5] = token6;	// mouse out action
		lpszBuffers[6] = token7;	// hotspot name
		lpszBuffers[7] = token8;	// hotspot status

		count = LCTokenize(args, lpszBuffers, 8, NULL);
		if(count >= 6)
		{
			g_Hotspots[i].stat = TRUE; // defaul to hotspot "on"
			g_Hotspots[i].name[0] = '\0'; 

			g_Hotspots[i].spot.left = atoi(token1);
			g_Hotspots[i].spot.top = atoi(token2);
			g_Hotspots[i].spot.right = atoi(token3);
			g_Hotspots[i].spot.bottom = atoi(token4);
			g_Hotspots[i].spot.left = CONVERT_COORDINATE_X(g_Hotspots[i].spot.left);
			g_Hotspots[i].spot.top = CONVERT_COORDINATE_Y(g_Hotspots[i].spot.top);
			g_Hotspots[i].spot.right = CONVERT_COORDINATE_WIDTH(g_Hotspots[i].spot.right);
			g_Hotspots[i].spot.bottom = CONVERT_COORDINATE_HEIGHT(g_Hotspots[i].spot.bottom);

			if(g_Hotspots[i].spot.right == 0)
				g_Hotspots[i].spot.right = g_rScreen.right;
			else 
				g_Hotspots[i].spot.right += g_Hotspots[i].spot.left;

			if(g_Hotspots[i].spot.bottom == 0)
				g_Hotspots[i].spot.bottom = g_rScreen.bottom;
			else
				g_Hotspots[i].spot.bottom += g_Hotspots[i].spot.top;

			StringCbCopy(g_Hotspots[i].in_action, sizeof(g_Hotspots[i].in_action), token5);
			StringCbCopy(g_Hotspots[i].out_action, sizeof(g_Hotspots[i].out_action), token6);
			
			if(count >= 7) // copy the name
        StringCbCopy(g_Hotspots[i].name, sizeof(g_Hotspots[i].name), token7);
			
			if(count == 8)
			{
				if(stricmp(token8, "on") == 0 || stricmp(token8, "true") == 0 || stricmp(token8, "yes") == 0)
				{
					g_Hotspots[i].stat = TRUE;
				}
				else if(stricmp(token8, "off") == 0 || stricmp(token8, "false") == 0 || stricmp(token8, "no") == 0)
				{
					g_Hotspots[i].stat = FALSE;
				}
			}
			
			if(i == g_Count)
				++g_Count;

			SystemAdjust();
		}
		else
		{
			//change it to a warning box
			MessageBox(NULL, "Invalid ckHotspots config found!", "ckHotspots", MB_OK);
		}
		
	}	


}

void RemoveHotspot(HotspotT *pSpot, void *NotUsed) { pSpot->stat = SLOT_EMPTY; }
void BangRemove(HWND caller, LPCSTR args)
{
	if(args && *args) /* don't remove all if none specified */ 
	{
    ForEachHotspot(args, &RemoveHotspot, NULL);
    SystemAdjust();
	}
}

void AdjustHotspot(HotspotT *pSpot, void *pAmounts)
{
  RECT *r = (RECT *)pAmounts;

  RECT rNew =
  {
    pSpot->spot.left + r->left,
    pSpot->spot.top + r->top,
    pSpot->spot.right + r->right,
    pSpot->spot.bottom + r->bottom
  };

	// is this new area on the screen? a little sanity checking.
  if (rNew.left < g_rScreen.left)
    rNew.left = g_rScreen.left;
  if (rNew.left > g_rScreen.right)
    rNew.left = g_rScreen.right;

  if (rNew.top < g_rScreen.top)
    rNew.top = g_rScreen.top;
  if (rNew.top > g_rScreen.bottom)
    rNew.top = g_rScreen.bottom;

  if (rNew.right < rNew.left)
    rNew.right = rNew.left;
  if (rNew.right > g_rScreen.right)
    rNew.right = g_rScreen.right;

  if (rNew.bottom < rNew.top)
    rNew.bottom = rNew.top;
  if (rNew.bottom > g_rScreen.bottom)
    rNew.bottom = g_rScreen.bottom;

  pSpot->spot = rNew;
}

// added by Eric Moore 2003-04-27 
void BangAdjust(HWND caller, LPCSTR args)
{
	// !HotspotAdjust name dx dy dw dh 
	char *buffers[5];
	char name[MAX_LINE_LENGTH], dx[MAX_LINE_LENGTH], dy[MAX_LINE_LENGTH],
		dw[MAX_LINE_LENGTH], dh[MAX_LINE_LENGTH], extra[MAX_LINE_LENGTH];
	name[0] = dx[0] = dy[0] = dw[0] = dh[0] = extra[0] = '\0';
	buffers[0] = name;
	buffers[1] = dx;
	buffers[2] = dy;
	buffers[3] = dw;
	buffers[4] = dh;

	if(LCTokenize(args, buffers, 5, extra) == 5)
	{
    RECT r = { atoi(dx), atoi(dy), atoi(dw), atoi(dh) };
    ForEachHotspot(name, &AdjustHotspot, &r);
	}
	else
	{
		MessageBox(NULL,"Incorrect number of arguments in !HotspotAdjust\nUsage:\n\t!HotspotAdjust name dx dy dw dh", "ckHotspots", MB_OK);
	}
}

void DebugHotspot(HotspotT *pSpot, void *pBuffer)
{
  char *buffer = (char *)pBuffer;
  const char *stat = "unknown";
  switch (pSpot->stat)
  {
  case TRUE: stat = "on"; break;
  case FALSE: stat = "off"; break;
  case SLOT_EMPTY: stat = "removed"; break;
  }

	StringCchPrintf(buffer, MAX_LINE_LENGTH,
    "Spot # %d : (%d, %d), (%d, %d)\nIn action:%s\nOut action:%s\nName:%s\nStatus:%s",
				(pSpot-g_Hotspots),
        pSpot->spot.left,  pSpot->spot.top,
				pSpot->spot.right, pSpot->spot.bottom,
				pSpot->in_action,  pSpot->out_action,
        pSpot->name,       stat);
	MessageBox(NULL, buffer, "ckHotspots", MB_OK|MB_TOPMOST);
}

void BangDebug(HWND caller, LPCSTR args)
{
	char buffer[4096];
  if (!ForEachHotspot(args, &DebugHotspot, buffer))
    MessageBox(NULL, "No matching hotspots found.", "ckHotspots", MB_OK|MB_TOPMOST);
}

///////////////////////////////
// Litestep module specific functions
int initModule(HWND parent, HINSTANCE dll, wharfDataType* wd)
{
	return initModuleEx(parent, dll, (wd)? wd->lsPath : NULL);	
}

int initModuleEx(HWND parent, HINSTANCE dll, LPCSTR szPath)
{
	//const char lpszClassName[] = "ckHotspots";
	
	FILE *f;
	
	char buffer[MAX_LINE_LENGTH];
	int Timerout = 100;
	WNDCLASS wc;

	//save screen dimension
  g_rScreen.left = SCREEN_LEFT;
  g_rScreen.top = SCREEN_TOP;
  g_rScreen.right = g_rScreen.left + SCREEN_WIDTH;
  g_rScreen.bottom = g_rScreen.top + SCREEN_HEIGHT;


	memset(&wc, 0, sizeof (wc));
	
	wc.lpfnWndProc = WindowProc;
	wc.hInstance = g_hInstance;
	wc.lpszClassName = CKHOTSPOTSCLASSNAME;
	
	//register window
	if(!RegisterClass(&wc))
	{
		MessageBox(NULL, "ckHotspots failed to register class", "Error", MB_OK);
		return -1; //failed	
	}

	//create window
	g_HotspotHWND = 
		CreateWindowEx(WS_EX_TOOLWINDOW, wc.lpszClassName, wc.lpszClassName, 
		0, 0, 0, 0, 0, NULL, NULL, dll, NULL);
	

	if(!g_HotspotHWND)
	{
		MessageBox(NULL, "ckHotspots failed to create window", "Error", MB_OK);
		return -1; //failed
	}

	//exclude ourselves from vwm
	SetWindowLong(g_HotspotHWND, GWL_USERDATA, magicDWord);
	
	//Which engine to employ
	g_Version1Style = GetRCBool("ckHotspotsVersion1Style", TRUE);
    g_MouseHookStyle = GetRCBool("ckHotspotsMouseHookStyle", TRUE);
    if(GetRCBool("IHaveAlotOfResourcesAndIReallyWantSomeVeryQuickStyle", TRUE))
    {
        g_Version1Style = g_MouseHookStyle = TRUE;
    }

	//read in all the configs
	f = LCOpen(NULL);
	if (f) 
	{	
		char *ConfigToken;
		
		while (LCReadNextConfig (f, "*ckHotspot", buffer, sizeof (buffer)))
		{
			//GetToken(buffer, NULL, &ConfigToken, FALSE);
			//the following will the the same thing as above function
			ConfigToken = buffer;
			while(!isspace(*ConfigToken++));
			
			while(isspace(*ConfigToken))
				++ConfigToken;

			BangAdd(g_HotspotHWND, ConfigToken);
		}
		LCClose(f);

		/*
		wsprintf(buffer, "Total %d spots", g_Count);
		MessageBox(NULL, buffer, "ckHotspots", MB_OK);
		*/
	}	

	//register message
	SendMessage(GetLitestepWnd(), LM_REGISTERMESSAGE, (WPARAM)g_HotspotHWND, (LPARAM)Msgs);

	//set up bangs
	AddBangCommand("!HotspotON", BangOn);
	AddBangCommand("!HotspotOFF", BangOff);
	AddBangCommand("!HotspotToggle", BangToggle);
	AddBangCommand("!HotspotAdd", BangAdd);
	AddBangCommand("!HotspotRemove", BangRemove);
	//Added by Eric Moore 2003-04-27
	AddBangCommand("!HotspotAdjust", BangAdjust);

	AddBangCommand("!HotspotDebug", BangDebug);

    // if user didn't specify anything, the default is using version1 style!
    if(!g_Version1Style && !g_MouseHookStyle)
    {
        g_Version1Style = TRUE;
    }

    if(g_MouseHookStyle)
	{
		//load library then initialize
		INITHOOK initHook = NULL;
		if(LoadHookSettings())
        {
		    initHook = (INITHOOK) GetProcAddress(g_HookInstance, "InitializeHook");
		    initHook(g_HotspotHWND, WM_CKMOUSEHOOKMSG);
        }
        else
        {
            g_MouseHookStyle = FALSE;
            MessageBox(NULL, "Default style will be used", "ckHotspots", MB_OK);
            g_Version1Style = TRUE;
        }
	}

    if(g_Version1Style)
	{//Set our timer
		Timerout = GetRCInt("ckHotspotsTimeOut", Timerout);
		g_Timer	= SetTimer(g_HotspotHWND, 1, Timerout, NULL);
		if(g_Timer == 0)
		{
			MessageBox(NULL, "ckHotspots failed to set timer", "Error", MB_OK);
            g_Version1Style = FALSE;
		}
	}

    if(!g_Version1Style && !g_MouseHookStyle)
    {
        //both style failed
        SystemOff();
        return -1;
    }

	SystemAdjust();

	return 0; // success
}

int quitModule(HINSTANCE dll)
{
	if(g_Timer != 0)
		KillTimer(g_HotspotHWND , 1);
	
	SystemOff();
	
	//set up bangs
	RemoveBangCommand("!HotspotON");
	RemoveBangCommand("!HotspotOFF");
	RemoveBangCommand("!HotspotToggle");
	RemoveBangCommand("!HotspotAdd");
	RemoveBangCommand("!HotspotRemove");
	//Added By Eric Moore 2003-04-27
	RemoveBangCommand("!HotspotAdjust");

	RemoveBangCommand("!HotspotsDebug");

	SendMessage(GetLitestepWnd(), LM_UNREGISTERMESSAGE, (WPARAM)g_HotspotHWND, (LPARAM)Msgs);

	DestroyWindow(g_HotspotHWND);
	UnregisterClass(CKHOTSPOTSCLASSNAME, g_hInstance);

	//MessageBox(NULL, "returning from quit", "ckHotspots", MB_OK);

	if(g_MouseHookStyle)
	{
		int i;
		//let's loop a bit to make sure it's unloaded
		for(i=0; i<10 || g_HookInstance != NULL; ++i)
			UnloadHookSettings();
		//we got a problem
		if(g_HookInstance != NULL)
			return 1;
	}

	return 0;
}

///////////////////////////////////
// helper functions

// turn on and off the hotspot system, when on, the system
// will check for mouse movement and stuff
void SystemOn(void)
{
	if(g_MouseHookStyle)
		SetMouseHook();
	g_Status = TRUE;
}

void SystemOff(void)
{
	g_Status = FALSE;
	if(g_MouseHookStyle)
		RemoveMouseHook();
}

// this function will adjust to turn system on or off
// depend on whether all hotspots or activated or deactivated
void SystemAdjust(void)
{
	int i;

  if(g_Version1Style || g_MouseHookStyle)
  {
    for (i=0; i<g_Count; ++i)
    {
      if (g_Hotspots[i].stat == TRUE)
      {
        /* found one that is on, don't need to keep looking */ 
        SystemOn();
        return;
      }
    }

    /* didn't find any on, turn it off */ 
    SystemOff();
  }
}

/////////////////////////////
// Hook functions

//our mouse call back function
/*
LRESULT CALLBACK MouseProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	MOUSEHOOKSTRUCT *ptr;
	//HWND ckHotspotsHWND;

	if(nCode < 0)
		return CallNextHookEx(g_Hook, nCode, wParam, lParam);
	
	//ckHotspotsHWND = FindWindow(CKHOTSPOTSCLASSNAME, NULL);
	if(g_HotspotHWND)
	{
		if(wParam == WM_MOUSEMOVE || wParam == WM_NCMOUSEMOVE)
		{
			ptr = (MOUSEHOOKSTRUCT *)lParam;
			PostMessage(g_HotspotHWND, WM_CKMOUSEHOOKMSG, ptr->pt.y, ptr->pt.x);
		}
	}

	return 0;
}
*/


//setup the mouse hook	
void SetMouseHook(void)
{
	if(SetMouseHookFunc != NULL)
		SetMouseHookFunc(TRUE);
}

void RemoveMouseHook(void)
{
	if(SetMouseHookFunc != NULL);
		SetMouseHookFunc(FALSE);
}

BOOL LoadHookSettings()
{
	if(g_HookInstance == NULL)
	{
    char ckHotspotsPath[MAX_PATH];
    char oldcwd[MAX_PATH];
    char *fslash, *bslash;

    /* allow ckMouseHook.dll to be in the same dir as this dll */ 
    GetModuleFileName(g_hInstance, ckHotspotsPath, MAX_PATH);
    GetCurrentDirectory(MAX_PATH, oldcwd);
    fslash = strrchr(ckHotspotsPath, '/');
    bslash = strrchr(ckHotspotsPath, '\\');
    /* if neither is found, they'll be == so no risk */ 
    if (fslash > bslash) *fslash = 0;
    else if (bslash > fslash) *bslash = 0;
    SetCurrentDirectory(ckHotspotsPath);
		g_HookInstance = LoadLibrary("ckMouseHook.dll");
    SetCurrentDirectory(oldcwd);

		if(g_HookInstance != NULL)
		{
			SetMouseHookFunc = (SETMOUSEHOOK) GetProcAddress(g_HookInstance, "SetMouseHook");
		}

		if(g_HookInstance == NULL || SetMouseHookFunc == NULL)
		{
			MessageBox(NULL, "ckMouseHook.dll failed to load, please make sure ckMouseHook.dll is in the same directory as litestep.exe and lsapi.dll", "ckHotspots", MB_OK);
		}
	}

    return (g_HookInstance != NULL && SetMouseHookFunc != NULL);
}

void UnloadHookSettings()
{
	if(g_HookInstance != NULL)
	{
		if(FreeLibrary(g_HookInstance))
		{
			SetMouseHookFunc = NULL;
			g_HookInstance = NULL;
		}

	}
}

////////////////////////////////////
// main entry function
//our DLL entry point
BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	UNREFERENCED_PARAMETER(lpvReserved);
	if(fdwReason == DLL_PROCESS_ATTACH)
	{
		g_hInstance = hinstDLL;
		DisableThreadLibraryCalls(g_hInstance);
	}
	return TRUE;
}


